#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import BackdropStyle as bs
from roller_constant_key import Option as ok
from roller_fu import desaturate, merge_layer_group
from roller_maya_style import Style, make_background
from roller_view_hub import get_canvas_points, invert_color, set_fill_context
from roller_view_real import add_sub_base_group, finish_style
import gimpfu as fu

pdb = fu.pdb


def make_style(v, maya):
    """
    Make a style layer.

    v: View
        Has scope variable.

    maya: Style
    Return: layer or None
        style layer
    """
    d = maya.value_d
    parent = add_sub_base_group(v, maya)
    z = make_background(v, d, parent)

    # RGBA, 'q'
    q = d[ok.COLOR_1A]

    # Use to set the opacity context for bucket fill.
    d[ok.FILL_OPACITY] = q[3] / 255. * 100.

    if d[ok.IDR][ok.INVERT]:
        q = invert_color(q)

    # fill point, 'x, y'
    x, y = get_canvas_points(d)[:2]

    # Preserve.
    foreground = pdb.gimp_context_get_foreground()

    set_fill_context(d)
    pdb.gimp_context_set_foreground(q)
    pdb.gimp_drawable_edit_bucket_fill(z, fu.FOREGROUND_FILL, x, y)

    # Restore.
    pdb.gimp_context_set_foreground(foreground)

    if d[ok.IDR][ok.DESATURATE]:
        desaturate(z)
    return finish_style(merge_layer_group(parent), "Color Fill")


class ColorFill(Style):
    """Create Backdrop Style output."""

    def __init__(self, *q):
        self.init_background(*q + (make_style,))

    def do(self, v, d, is_change):
        """
        Produce output.

        v: View
        d: dict
            Backdrop Style Preset

        is_change: bool
            Is the state of the super Maya's matter and/or mask.
        """
        self.is_dependent = \
            d[ok.BRW][ok.BACKGROUND][ok.TYPE] == bs.BACKDROP_IMAGE
        super(ColorFill, self).do(v, d, is_change)
