#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import BackdropStyle as bs
from roller_constant_key import Option as ok
from roller_fu import invert_and_desaturate, merge_layer_group
from roller_maya_style import Style, make_background
from roller_view_hub import (
    get_canvas_points, set_fill_context, set_gimp_pattern
)
from roller_view_real import add_sub_base_group, finish_style
import gimpfu as fu

pdb = fu.pdb


def make_style(v, maya):
    """
    Make the Backdrop Style.

    v: View
    maya: PatternFill
    Return: layer
        with the style material
    """
    d = maya.value_d
    parent = add_sub_base_group(v, maya)
    z = make_background(v, d, parent)

    # fill point, 'x, y'
    x, y = get_canvas_points(d)[:2]

    set_fill_context(d)
    set_gimp_pattern(d[ok.PATTERN])
    pdb.gimp_drawable_edit_bucket_fill(z, fu.FILL_PATTERN, x, y)
    invert_and_desaturate(d[ok.IDR], z)

    z = merge_layer_group(parent)
    return finish_style(z, "Pattern Fill")


class PatternFill(Style):
    """Create Backdrop Style output."""

    def __init__(self, *q):
        self.init_background(*q + (make_style,))

    def do(self, v, d, is_change):
        """
        Produce output.

        v: View
        d: dict
            Backdrop Style Preset

        is_change: bool
            Is the state of the super Maya's matter and/or mask.
        """
        self.is_dependent = \
            d[ok.BRW][ok.BACKGROUND][ok.TYPE] == bs.BACKDROP_IMAGE
        super(PatternFill, self).do(v, d, is_change)
