#!/usr/bin/env python
# -*- coding: utf-8 -*-
from roller_constant_for import Shape as sh
from roller_model_face import make_h_face, make_v_face
from roller_one_rect import Rect


def bottom_arrange(q, w, h, center, a):
    """
    Is a left side mirrored polygon for the Bottom Box Type.

    q: tuple
        x, y float series, eight values defining four vertices
        cell shape

    w, h: float
        Is the size of the Face as a right rectangle.

    center: tuple
        (x, y) of float
        Is the center of the Shift rectangle.

    a: Rect
        cell Shift rectangle
    """
    x, y = q[2], q[3]
    x1, y1 = x + w, y + h
    a.merged = Rect(x, y, w, h)
    a.form = x, y, x1, y, x1, y1, x, y1
    a.shape = (
        q[0], q[1],
        x, y,
        x, center[1],
        q[10], q[11]
    )
    a.foam = (
        q[0], q[1],
        x, y,
        q[10], q[11],
        x, center[1]
    )


def left_arrange(q, w, h, center, a):
    """Is a top side mirrored polygon for the Left Box Type."""
    x, y = q[2], q[3]
    x1, y1 = x + w, y + h
    a.merged = Rect(x, y, w, h)
    a.form = x, y, x1, y, x1, y1, x, y1
    a.shape = (
        x, y,
        q[4], q[5],
        q[6], q[7],
        center[0], q[1]
    )
    a.foam = (
        x, y,
        q[4], q[5],
        center[0], q[1],
        q[6], q[7]
    )


def right_arrange(q, w, h, center, a):
    """Is a top side mirrored polygon for the Right Box Type."""
    x, y = q[0], q[3]
    x1, y1 = x + w, y + h
    a.merged = Rect(x, y, w, h)
    a.form = x, y, x1, y, x1, y1, x, y1
    a.shape = (
        x, q[1],
        q[2], y,
        q[4], q[5],
        center[0], q[1]
    )
    a.foam = (
        q[2], y,
        q[4], q[5],
        x, q[1],
        center[0], q[1]
    )


def top_arrange(q, w, h, center, a):
    """Is a left side mirrored polygon for the Top Box Type."""
    x, y = q[0], q[1]
    x1, y1 = x + w, y + h
    a.merged = Rect(x, y, w, h)
    a.form = x, y, x1, y, x1, y1, x, y1
    a.shape = (
        x, y,
        q[2], center[1],
        q[8], q[9],
        q[10], q[11]
    )
    a.foam = (
        x, y,
        q[8], center[1],
        q[10], q[11],
        q[8], q[9]
    )


FACE1_BUILD_D = {
    sh.BOTTOM: bottom_arrange,
    sh.LEFT: left_arrange,
    sh.RIGHT: right_arrange,
    sh.TOP: top_arrange
}
FACE1_TRANSFORM_D = {
    sh.BOTTOM: make_h_face,
    sh.LEFT: make_v_face,
    sh.RIGHT: make_v_face,
    sh.TOP: make_h_face
}
